using System;
using System.IO;
using System.Collections;
using Server;
using Server.Items;
using Server.Mobiles;

namespace Server.Mobiles
{
	[CorpseName( "a gothic corpse" )]
	public class Ashieva : BaseCreature
	{
		private static ArrayList m_VampireList;
		
		[Constructable]
		public Ashieva(): base( AIType.AI_Melee, FightMode.Closest, 10, 1, 0.2, 0.4 ) 
		{
			Name = "Ashieva";
			Title = "the Vampire Queen";
			Body = 0x191;
			Hue = 0x847e;
			Female = true;
			SpeechHue = 1157;

			SetStr( 350, 450 );
			SetDex( 280, 300 );
			SetInt( 626, 681 );
			
			SetStam( 100, 150 );
			SetMana( 1200, 2000 );

			SetHits( 5000, 7000 );

			SetDamage( 20, 28 );

			SetDamageType( ResistanceType.Physical, 50 );
			SetDamageType( ResistanceType.Poison, 50 );

			SetResistance( ResistanceType.Physical, 70, 80 );
			SetResistance( ResistanceType.Fire, 60, 70 );
			SetResistance( ResistanceType.Cold, 20, 30 );
			SetResistance( ResistanceType.Poison, 80, 90 );
			SetResistance( ResistanceType.Energy, 40, 50 );
			
			SetSkill( SkillName.MagicResist, 35.1, 60.0 );
			SetSkill( SkillName.Tactics, 35.1, 80.0 );
			SetSkill( SkillName.Wrestling, 95.1, 120.0 );
			SetSkill( SkillName.Magery, 110.3, 138.0 );
			SetSkill( SkillName.Meditation, 95.1, 117.0 );
			SetSkill( SkillName.Focus, 95.7, 121.2 );
			SetSkill( SkillName.Necromancy, 117.1, 152.0 );
			SetSkill( SkillName.SpiritSpeak, 89.1, 92.0 );

			Fame = 21450;
			Karma = -21450;

			VirtualArmor = 48;
			
			AddItem( ItemSet( new HoodedShroudOfShadows( Hue = 1150) ) );
			AddItem( ItemSet( new Bandana() ) );
			AddItem( ItemSet2( new BoneLegs() ) );
			AddItem( ItemSet2( new Sandals() ) );
			AddItem( ItemSet2( new Cloak() ) );
			AddItem( ItemSet2( new LeatherGloves() ) );
			AddItem( new LongHair(38) );
			
			PackGold( 3266, 4589 );
                         
			
			PackMagicItems( 1, 4, 0.80, 0.75 );
			PackMagicItems( 3, 6, 0.60, 0.45 );
			PackMagicItems( 4, 7, 0.60, 0.85 );
			PackMagicItems( 3, 5, 0.50, 0.95 );
			PackMagicItems( 2, 3, 0.20, 1 );
			PackMagicItems( 3, 7, 0.62, 0.65 );
			AddLoot( LootPack.FilthyRich, 4 );

			PackNecroScroll( 9 ); // Poison Strike
			PackNecroScroll( 14 ); // Wither
			PackNecroScroll( 10 ); // Strangle
			PackNecroScroll( 6 ); // Lich Form
			PackNecroScroll( 13 ); // Vengeful Spirit
			
			new TransformCheck(this).Start();
			new VampireDisappear(this).Start();
			
			m_VampireList = new ArrayList();
		}
		
		public static Item ItemSet( Item item )
		{
			item.Movable = false;
			item.Hue = 1175;
			
			return item;
		}
		
		public static Item ItemSet2( Item item2 )
		{
			item2.Movable = false;
			item2.Hue = 1157;
			
			return item2;
		}
		
		public override void OnGaveMeleeAttack( Mobile defender )
		{
			base.OnGaveMeleeAttack( defender );
			
			if ( 0.5 >= Utility.RandomDouble() )
			{
				BeginBleed( defender );
			}
			
			if ( 0.05 >= Utility.RandomDouble() )
			{
				Say( "You Fool! I shall swarm my babies to you!" );
				SpawnBabies( defender );
			}
		}
		
		private static Hashtable m_Table = new Hashtable();

		public static bool IsBleeding( Mobile m )
		{
			return m_Table.Contains( m );
		}

		public static void BeginBleed( Mobile m )
		{
			m.Damage( Utility.RandomMinMax( 2, 5 ) );
			m.Emote( "*the pain of blood has departed {0} body*", m.Female ? "her" : "his" );
			m.PlaySound( 0x133 );
			m.FixedParticles( 0x377A, 244, 25, 9950, 31, 0, EffectLayer.Waist );
			m.SendLocalizedMessage( 1060160 ); // You are bleeding!	
			
			Timer t = (Timer)m_Table[m];

			if ( t != null )
				t.Stop();

			t = new InternalTimer( m );
			m_Table[m] = t;

			t.Start();
		}

		public static void DoBleed( Mobile m, int level )
		{
			if ( m.Alive )
			{
				int damage = Utility.RandomMinMax( level, level * 2 );

				if ( !m.Player )
					damage *= 2;

				m.PlaySound( 0x133 );
				m.Damage( damage );

				Blood blood = new Blood();

				blood.ItemID = Utility.Random( 0x122A, 5 );

				blood.MoveToWorld( m.Location, m.Map );
			}
			else
			{
				EndBleed( m, false );
			}
		}

		public static void EndBleed( Mobile m, bool message )
		{
			Timer t = (Timer)m_Table[m];

			if ( t == null )
				return;

			t.Stop();
			m_Table.Remove( m );

			m.SendLocalizedMessage( 1060167 ); // The bleeding wounds have healed, you are no longer bleeding!
		}
		
		public override void OnGotMeleeAttack( Mobile attacker )
		{
			base.OnGotMeleeAttack( attacker );
			
			if ( 0.05 >= Utility.RandomDouble() )
			{
				BeginBleed( attacker );
				attacker.SendMessage( "{0} Counterattacks you!", this.Name );
			}
			
			if ( 0.1 >= Utility.RandomDouble() )
			{
				Say( "You Fool! I CALL ON MY CHILDREN" );
				SpawnBabies( attacker );
			}
		}
                 
public override void OnDeath( Container c )
		{		
		c.DropItem( new Tokens( 1000 ) );
c.DropItem( ItemSet( new DeathShroud() ) );
base.OnDeath( c );
		}
		

		public override bool OnBeforeDeath()
		{	
			ArrayList eachvamp = m_VampireList;
			
			foreach ( Mobile m in eachvamp )
			{	
				YoungVampire vamp = m as YoungVampire;			
				if( vamp.Alive && vamp.ControlMaster == this )
				{
					vamp.Kill();
					vamp.Emote( "*Mine*" );
					return base.OnBeforeDeath();
				}
			}
			
			return base.OnBeforeDeath();
		}
		
		public void SpawnBabies( Mobile attacker )
		{
			Map map = this.Map;

			if ( map == null )
				return;

			int newVampires = Utility.RandomMinMax( 2, 5 );

			for ( int i = 0; i < newVampires; ++i )
			{
				YoungVampire vamp = new YoungVampire();

				vamp.Team = this.Team;
				vamp.FightMode = FightMode.Closest;

				bool validLocation = false;
				Point3D loc = this.Location;

				for ( int j = 0; !validLocation && j < 10; ++j )
				{
					int x = X + Utility.Random( 5 ) - 1;
					int y = Y + Utility.Random( 5 ) - 1;
					int z = map.GetAverageZ( x, y );

					if ( validLocation = map.CanFit( x, y, this.Z, 16, false, false ) )
						loc = new Point3D( x, y, Z );
					else if ( validLocation = map.CanFit( x, y, z, 16, false, false ) )
						loc = new Point3D( x, y, z );
				}

				vamp.MoveToWorld( loc, map );
				vamp.Controled = true;
				vamp.ControlMaster = this;
				vamp.ControlTarget = this;
				vamp.ControlOrder = OrderType.Guard;
				vamp.Combatant = attacker;
				
				m_VampireList.Add( vamp );
			}
		}
		
		public override bool IsEnemy( Mobile m )
		{
			YoungVampire vamp = m as YoungVampire;
			
			if ( m is YoungVampire && vamp.Controled && vamp.ControlMaster == this )
				return false;

			return base.IsEnemy( m );
		}
		
		public override Poison PoisonImmune{ get{ return Poison.Deadly; } }
		public override bool AlwaysMurderer{ get{ return true; } }
		public override bool ShowFameTitle{ get{ return false; } }
		public override bool BardImmune{ get{ return true; } }
		public override bool CanRummageCorpses{ get{ return true; } }
		
		private class InternalTimer : Timer
		{
			private Mobile m_Mobile;
			private int m_Count;

			public InternalTimer( Mobile m ) : base( TimeSpan.FromSeconds( 2.0 ), TimeSpan.FromSeconds( 2.0 ) )
			{
				m_Mobile = m;
				Priority = TimerPriority.TwoFiftyMS;
			}

			protected override void OnTick()
			{
				DoBleed( m_Mobile, 5 - m_Count );

				if ( ++m_Count == 5 )
					EndBleed( m_Mobile, true );
			}
		}
		
		private class TransformCheck : Timer
		{
			private Ashieva m_Vampire;
			private static bool AlreadySaid;
			private static bool PlayOnce;
			
			public TransformCheck( Ashieva vampire ): base( TimeSpan.FromSeconds( 0.1 ), TimeSpan.FromSeconds( 0.1 ) )
			{
				m_Vampire = vampire;
			}
			
			protected override void OnTick()
			{
				if( !m_Vampire.Alive )
					Stop();
					
				if( m_Vampire.Body == 0x95 && !AlreadySaid )
				{
					m_Vampire.Say( "I shall show you who I really AM!" );
					AlreadySaid = true;
				}
					
				if( m_Vampire.Hits <= 500 )
				{
					m_Vampire.Body = 0x95;
					m_Vampire.Hue = 133;
					Point3D loc = m_Vampire.Location;
					
					if( !PlayOnce )
					{
						
						
						m_Vampire.RawStr = 250 + m_Vampire.RawStr;
						m_Vampire.RawInt = 250 + m_Vampire.RawInt;
						m_Vampire.RawDex = 50 + m_Vampire.RawDex;
						m_Vampire.Mana = m_Vampire.Mana - 450;
						
						PlayOnce = true;
					}
				}
				else
				{
					m_Vampire.Body = 0x191;
					m_Vampire.Hue = 0x847e;
					
					m_Vampire.RawStr = m_Vampire.RawStr - 250;
					m_Vampire.RawInt = m_Vampire.RawInt - 250;
					m_Vampire.RawDex = m_Vampire.RawDex - 50;
					
					PlayOnce = false;
					
					if( m_Vampire.Hits >= 2000 )
						AlreadySaid = false;
				}
			}
		}
		
		private class VampireDisappear : Timer
		{
			private Ashieva m_Vampire;
			
			public VampireDisappear( Ashieva vampire ): base( TimeSpan.FromMinutes( 1.0 ), TimeSpan.FromMinutes( 1.0 ) )
			{
				m_Vampire = vampire;
			}
			
			protected override void OnTick()
			{
				if( m_Vampire.Combatant == null && m_Vampire.Followers != 0 )
				{
					ArrayList eachvamp = m_VampireList;;
					
					m_Vampire.Say( "I shall send my childrens back to the underworld!" );
								
					foreach ( Mobile m in eachvamp )
					{	
						YoungVampire vamp = m as YoungVampire;			
						if( vamp.ControlMaster == m_Vampire )
						{							
							Effects.PlaySound( vamp, m_Vampire.Map, 0x307 );
	
							Effects.SendLocationEffect( new Point3D( vamp.X, vamp.Y, vamp.Z ), vamp.Map, 0x3728, 15 );
							vamp.Delete();
						}
					}
				}
			}
		}
		
		public Ashieva( Serial serial ) : base( serial )
		{
			new TransformCheck(this).Start();
			new VampireDisappear(this).Start();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
			
			writer.WriteMobileList( m_VampireList );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
			
			m_VampireList = reader.ReadMobileList();
		}
	}
}